import greenfoot.*;
import java.awt.Color;

/**
 * The Code of the matrix themed background
 * 
 * @author 
 * @version 
 */
public class Background extends JuniorExerciseTwoActor
{
    /** Store the colour used of this code **/
    private Color colour;
    
    /** Store the speed of which the code falls **/
    private int SPEED = 3;
    
    /** Store the amount of codes created by this code **/
    private int MODIFIIER = 1;
    
    /** If this code is the leader of others that are created **/
    private boolean IS_LEADER;
    
    /** If the code is currently valid or not **/
    private boolean VALID = true;
    
    /** Store the characters string code **/
    private int raw;
    
    /** Store when the last action was held **/
    private long LAST_ACTION = System.currentTimeMillis();
    
    /**
     * Append a new peice of coding to the world
     * @param colour The colour used for the code
     * @param IS_LEADER if the code is the leader of others
     */
    public Background(Color colour, boolean IS_LEADER)
    {
        this.colour = colour;
        this.IS_LEADER = IS_LEADER;
        this.raw = '0' + (int)(Math.random() * 2);
        setImage(new GreenfootImage(10, 10));
    }
    
    /**
     * lets the code 'fall' down
     */
    public void act()
    {
        fall();
        draw();
    }
    
    /**
     * Used to draw the code onto the image
     */
    public void draw()
    {
        /**
         * Don't attempt to repaint if the object is 'invalid'
         */
        if (VALID) {
            GreenfootImage image = new GreenfootImage(15, 15);
            image.setColor(getColour());
            image.drawString(Character.toString((char) raw), 5, getImage().getHeight());
            setImage(image);
            duplicate();
        }
    }
    
    /**
     * Creates a follower onto this leading code
     */
    public void duplicate()
    {
        if(IS_LEADER && System.currentTimeMillis() - LAST_ACTION > JuniorExerciseTwo.TIME / 2) {
            Background newCode = new Background(JuniorExerciseTwo.CODE_COLOUR, false);
            getWorld().addObject(newCode, getX(), getY() - (MODIFIIER++ * 20));
            LAST_ACTION = System.currentTimeMillis();
        }
    }
    
    /**
     * Used to get the colour of the text depending on
     * the distance from the start to the end of the journey
     * @return Returns the colour of the falling code
     */
    public Color getColour()
    {
        int i = ((getY() / 10) * 7);
        int alpha = i > 255 ? 255 : i;
        return new Color(colour.getRed(), colour.getGreen(), colour.getBlue(), IS_LEADER ? 200 : alpha);
    }
    
    /**
     * Remove this object from the worlds list
     */
    public void remove()
    {
        VALID = false;
        getWorld().removeObject(this);
    }
    
    /**
     * Makes the code fall down until it reaches the edge
     */
    public void fall()
    {
        /**
         * Once the code reaches the maximum height it will be removed
         */
        if(getY() >= getWorld().getHeight() - 5) {
            remove();
        } else {
            /**
             * If the code is in contact with another it will be removed
             */
            if(getOneObjectAtOffset(0, getImage().getHeight(), Background.class) == null)
                setLocation(getX(), getY() + SPEED);
            else {
                remove();
            }
        }
    }
}