import greenfoot.*;
import java.awt.Color;
import java.awt.Font;
/**
 * An actor for text input.
 * 
 * @author 
 * @version 
 */
public class TextField extends JuniorExerciseTwoActor
{
    private GreenfootImage beginImage;
    private JuniorExerciseTwo responder;
    private String text;
    private String prefix;

    /**
     * Create a new text field with a width of 400.
     */
    public TextField()
    {
        this(400);
    }

    /**
     * Create a new text field with a given width.
     */
    public TextField(int width)
    {
        GreenfootImage img = new GreenfootImage(width, 50);
        img.setColor(Color.WHITE);
        img.fill();
        img.setColor(Color.WHITE);
        img.fillRect(3,3, img.getWidth() -6, img.getHeight() -6);
        img.setTransparency(150);
        setImage(img);
        this.beginImage = img;
        this.text = "";
        this.prefix = "";
    }

    /**
     * Sets the prefix of the text field (the text you see before
     * you typed anything).
     * @param prefix the prefix of the Textfield
     */
    public void setPrefix(String prefix)
    {
        if (!this.prefix.equals(prefix)) {
            this.prefix = prefix;
            showPrefix();
        }
    }

    /**
     * Sets the responder of the text field.
     * @param world the Class which is the Responder
     */
    public void setResponder(JuniorExerciseTwo world)
    {
        this.responder = world;
    }

    /**
     * Returns the responder of the text field.
     */
    public JuniorExerciseTwo getResponder()
    {
        return this.responder;
    }

    /**
     * Returns the text of the text field.
     * @return Text in the TextField
     */
    public String getText()
    {
        if (this.text != null) return this.text;
        return "";
    }

    /**
     * Returns the image of the text field, without the text.
     * @return the Image without text
     */
    public GreenfootImage getBeginImage()
    {
        return this.beginImage;
    }

    /**
     * Add text onto the text field if needed.
     */
    public void act() 
    {
        addInput();
    }

    /**
     * adds the character which is pressed
     */
    private void addInput()
    {
        String key = Greenfoot.getKey();
        if (key == null) return;
        if (key.equals("enter")) {
            if (this.responder != null)
                this.responder.textFieldIsDoneWithText(this.text, this);
        } else {
            if (key.equals("backspace")){
                if (text.length() > 0) {
                    text = text.substring(0, text.length() -1);
                }
            } else {
                if(key.equals("space")){
                    key = " ";
                }
                if (key.equals("shift") ||
                key.equals("control") ||
                key.equals("tab") ||
                key.equals("escape") ||
                (char)127 == key.charAt(0)) {
                    key = "";
                }
                text = text + key;
            }
        }
        refresh();
    }

    /**
     * Redraw the image of the text field.
     */
    public void refresh()
    {
        if (this.text.length() <= 0) {
            showPrefix();
        } else {
            GreenfootImage img = new GreenfootImage(beginImage);
            GreenfootImage txtImg = new GreenfootImage(this.text, 40, null, null);
            int x = 10;
            int y = img.getHeight() /2 - txtImg.getHeight() /2;
            img.drawImage(txtImg, x,y);
            setImage(img);
        }
    }

    /**
     * Writes the prefix in the Textfield if no text is in the TextField
     */
    private void showPrefix()
    {
        if (text.length() == 0) {
            GreenfootImage newImg = new GreenfootImage(this.beginImage);
            GreenfootImage img = new GreenfootImage(this.prefix, 40, Color.GRAY, null);
            int y = getImage().getHeight() /2 - img.getHeight() /2;
            newImg.drawImage(img, 10, y);
            setImage(newImg);
        }
    }
}